package scenariodiagrameditor.features;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.graphiti.features.ICreateFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateContext;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.Diagram;

import scenariodiagrameditor.util.Util;
import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioDiagram;
import de.uks.se.scenariodiagram.ScenarioLink;
import de.uks.se.scenariodiagram.ScenarioObject;
import de.uks.se.scenariodiagram.ScenariodiagramFactory;

public class CreateScenarioObjectFeature extends AbstractScenarioCreateFeature implements ICreateFeature {

	private final String DEFAULT_TAG = "scenarioObjectText";
	private final String DEFAULT_VALUE = "ScenarioObject";
	private final String DEFAULT_TYPE = "DefaultType";
	
	private final String DEFAULT_LINK_TAG = "scenarioLinkName";
	private final String DEFAULT_LINK_VALUE = "has";

	public CreateScenarioObjectFeature(IFeatureProvider fp) {
		super(fp, "ScenarioObject", "Creates a new ScenarioObject");
	}

	@Override
	public boolean canCreate(ICreateContext context) {
		ContainerShape targetContainer = context.getTargetContainer();
		
		if(targetContainer instanceof Diagram){
			return true;
		}
		else if(getBusinessObjectForPictogramElement(targetContainer) instanceof ScenarioObject){
			return true;
		}
		return false;
	}

	@Override
	public Object[] create(ICreateContext context) {
		ScenarioObject newScenarioObject = ScenariodiagramFactory.eINSTANCE.createScenarioObject();
	
		ScenarioAttribute newScenarioAttribute = ScenariodiagramFactory.eINSTANCE.createScenarioAttribute();
		newScenarioAttribute.setTag(DEFAULT_TAG);
		newScenarioAttribute.setValue(DEFAULT_VALUE);
		newScenarioObject.getAttributes().add(newScenarioAttribute);
		
		createEcoreStructure(newScenarioObject, newScenarioAttribute);
		
		ContainerShape targetContainer = context.getTargetContainer();
		
		if(getBusinessObjectForPictogramElement(targetContainer) instanceof ScenarioObject)
		{
			ScenarioObject targetObject = (ScenarioObject) getBusinessObjectForPictogramElement(targetContainer);
			targetObject.getContainedScenarioObjects().add(newScenarioObject);
			
			//create link
			ScenarioLink newScenarioLink = ScenariodiagramFactory.eINSTANCE.createScenarioLink();
			newScenarioLink.setSource(newScenarioObject);
			newScenarioLink.setTarget(targetObject);
			getDiagram().eResource().getContents().add(newScenarioLink);
			
			ScenarioAttribute nameAttribute = ScenariodiagramFactory.eINSTANCE.createScenarioAttribute();
			nameAttribute.setTag(DEFAULT_LINK_TAG);
			nameAttribute.setValue(DEFAULT_LINK_VALUE);
			newScenarioLink.getAttributes().add(nameAttribute);
			
			createEcoreReferences(newScenarioObject, targetObject, newScenarioLink);
		}
		else if (targetContainer instanceof Diagram)
		{
			getOrCreateScenarioDiagramFromDiagram(getDiagram()).getContainedScenarioObjects().add(newScenarioObject);
		}
		
		addGraphicalRepresentation(context, newScenarioObject);

		return new Object[] { newScenarioObject };
	}

	private void createEcoreReferences(ScenarioObject newScenarioObject, ScenarioObject targetObject, ScenarioLink newScenarioLink)
	{
		EReference outgoingRef = EcoreFactory.eINSTANCE.createEReference();
		EClass sourceObjectType = newScenarioObject.getObjectType();
		EReference incomingRef = EcoreFactory.eINSTANCE.createEReference();
		EClass targetObjectType = targetObject.getObjectType();
		
		// outgoing
		outgoingRef.setDefaultValueLiteral(DEFAULT_LINK_VALUE);
		sourceObjectType.getEStructuralFeatures().add(outgoingRef);//frs Speichern
		outgoingRef.setEType(targetObjectType);
		newScenarioLink.setOutgoingReference(outgoingRef);
		
		//opposites
		outgoingRef.setEOpposite(incomingRef);
		incomingRef.setEOpposite(outgoingRef);
		
		// incoming
		incomingRef.setDefaultValueLiteral(DEFAULT_LINK_VALUE);
		targetObjectType.getEStructuralFeatures().add(incomingRef);//fr's speichern
		incomingRef.setEType(sourceObjectType);
		newScenarioLink.setIncomingReference(incomingRef);
	}

	private void createEcoreStructure(ScenarioObject newScenarioObject, ScenarioAttribute newScenarioAttribute) {
		//find or create DefaultType
		EClass defaultScenarioObjectType = null;
		
		ScenarioDiagram scenarioDiagram = getOrCreateScenarioDiagramFromDiagram(getDiagram());
		for (EClass tmpType : scenarioDiagram.getObjectTypes())
		{
			//existing?
			if(tmpType.getInstanceTypeName().equals(DEFAULT_TYPE))
			{
				defaultScenarioObjectType = tmpType;
				break;
			}
		}
		
		//new
		if(defaultScenarioObjectType == null)
		{
			defaultScenarioObjectType = EcoreFactory.eINSTANCE.createEClass();
			defaultScenarioObjectType.setInstanceTypeName(DEFAULT_TYPE);
			newScenarioObject.setObjectType(defaultScenarioObjectType);
			scenarioDiagram.getObjectTypes().add(defaultScenarioObjectType); //frs Speichern
			
			EAttribute defaultTextAttribute = EcoreFactory.eINSTANCE.createEAttribute();
			defaultScenarioObjectType.getEStructuralFeatures().add(defaultTextAttribute); //frs Speichern
			newScenarioAttribute.setAttributeType(defaultTextAttribute);
			defaultTextAttribute.setName(DEFAULT_TAG);
			defaultTextAttribute.setEType(Util.getType(""));
		}
		//existing
		else
		{
			newScenarioObject.setObjectType(defaultScenarioObjectType);
			
			EAttribute defaultTextAttribute = null;
			for(EAttribute tmpAttribute : defaultScenarioObjectType.getEAttributes())
			{
				if(tmpAttribute.getName().equals(DEFAULT_TAG))
				{
					defaultTextAttribute = tmpAttribute;
					break;
				}
			}
			
			newScenarioAttribute.setAttributeType(defaultTextAttribute);
		}
	}
}
