package scenariodiagrameditor.features;

import java.util.HashSet;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.impl.AbstractDirectEditingFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;

import scenariodiagrameditor.util.Util;
import scenariodiagrameditor.util.Util.ScenarioDiagramTypes;
import de.uks.se.scenariodiagram.ScenarioAttribute;

public class DirectEditScenarioAttributeFeature extends AbstractDirectEditingFeature{
	
	public DirectEditScenarioAttributeFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public int getEditingType() {
		return TYPE_TEXT;
	}

	@Override
	public String getInitialValue(IDirectEditingContext context) {
		PictogramElement pe = context.getPictogramElement();
        ScenarioAttribute scenarioAttribute = (ScenarioAttribute) getBusinessObjectForPictogramElement(pe);
        String instanceTypeName = scenarioAttribute.getAttributeType().getEType().getInstanceTypeName();
		String initialValue = scenarioAttribute.getTag() + "=" + scenarioAttribute.getValue() + ":" + instanceTypeName;
        
        return initialValue;
	}
	
	@Override
	public boolean canDirectEdit(IDirectEditingContext context)
	{
        PictogramElement pe = context.getPictogramElement();
        Object object = getBusinessObjectForPictogramElement(pe);
        GraphicsAlgorithm ga = context.getGraphicsAlgorithm();

        if (object instanceof ScenarioAttribute && ga instanceof Text)
        {
            return true;
        }

        return false;
	}

	@Override
	public void setValue(String value, IDirectEditingContext context)
	{
        PictogramElement pe = context.getPictogramElement();
        ScenarioAttribute scenarioAttribute = (ScenarioAttribute) getBusinessObjectForPictogramElement(pe);
        
        String[] tagValue = value.split("=");
        if(tagValue.length > 1 &&
           (scenarioAttribute.getObject() != null && scenarioAttribute.getObject().getObjectType() != null ||
            scenarioAttribute.getLink() != null && scenarioAttribute.getLink().getSource().getObjectType() != null))
        {
        	updateModels(scenarioAttribute, tagValue);
        	
        	Text text = (Text)context.getGraphicsAlgorithm();
            text.setValue(value);
     
            updatePictogramElement(((Shape) pe).getContainer());
            
            getDiagramBehavior().refreshContent();
        }
	}

	private void updateModels(ScenarioAttribute scenarioAttribute, String[] tagValue)
	{
		scenarioAttribute.setAttributeType(null);
		
		String attrTag = tagValue[0];
		String attrValueType = tagValue[1];
		// already existing?
		if(scenarioAttribute.getObject() != null)
		{
			for(EAttribute tmpAttribute : scenarioAttribute.getObject().getObjectType().getEAttributes())
			{
				if(tmpAttribute.getName().equals(attrTag))
				{
					scenarioAttribute.setAttributeType(tmpAttribute);
					break;
				}
			}
		}
		else 
		{
			for(EAttribute tmpAttribute : scenarioAttribute.getLink().getSource().getObjectType().getEAttributes())
			{
				if(tmpAttribute.getName().equals(attrTag))
				{
					scenarioAttribute.setAttributeType(tmpAttribute);
					break;
				}
			}
		}
		
		// new attribute and no type defined -> type String
		EAttribute existingAttributeType = scenarioAttribute.getAttributeType();
		if(existingAttributeType == null && !tagValue[1].contains(":"))
		{
			//genModel
			EAttribute newAttributeType = EcoreFactory.eINSTANCE.createEAttribute();
			newAttributeType.setName(attrTag);
			newAttributeType.setEType(Util.getType(""));
			
			if(scenarioAttribute.getObject() != null)
			{				
				EClass objectType = scenarioAttribute.getObject().getObjectType();
				objectType.getEStructuralFeatures().add(newAttributeType); //frs Speichern
			}
			else
			{
				EClass objectType = scenarioAttribute.getLink().getSource().getObjectType();
				objectType.getEStructuralFeatures().add(newAttributeType);
			}
		
			//viewModel
			scenarioAttribute.setTag(attrTag);
			scenarioAttribute.setValue(attrValueType);
			scenarioAttribute.setAttributeType(newAttributeType);
		}
		// new attribute and type defined
		else if(existingAttributeType == null && tagValue[1].contains(":"))
		{
			//genModel
			EAttribute newAttributeType = EcoreFactory.eINSTANCE.createEAttribute();
			newAttributeType.setName(attrTag);
			
			EClass objectType = scenarioAttribute.getObject().getObjectType();
			objectType.getEStructuralFeatures().add(newAttributeType); //frs Speichern
			
			String[] valueType = attrValueType.split(":");
			if(valueType.length > 1)
			{
				String attrValue = valueType[0];
				String attrType = valueType[1];
				
				//viewModel
				scenarioAttribute.setTag(attrTag);
				scenarioAttribute.setValue(attrValue);
				scenarioAttribute.setAttributeType(newAttributeType);
				
				//genModel
				newAttributeType.setEType(Util.getType(attrType.toUpperCase()));
			}
		}
		//existing attribute and no type defined -> type String
		else if(existingAttributeType != null && !tagValue[1].contains(":"))
		{
			//viewModel
			scenarioAttribute.setTag(attrTag);
			scenarioAttribute.setValue(attrValueType);			
			
			//genModel
			existingAttributeType.setEType(Util.getType(""));
		}
		//existing attribute and type defined
		else
		{
			String[] valueType = attrValueType.split(":");
			if(valueType.length > 1)
			{
 
				String attrValue = valueType[0];
				String attrType = valueType[1];
				//viewModel
				scenarioAttribute.setTag(attrTag);
				scenarioAttribute.setValue(attrValue);
				
				//genModel
				existingAttributeType.setEType(Util.getType(attrType.toUpperCase()));
			}
		}
	}
	
	@Override
	public boolean isAutoCompletionEnabled() {
		return true;
	}

	@Override
	public boolean isCompletionAvailable() {
		return true;
	}

	@Override
	public String[] getValueProposals(String value, int caretPos,
			IDirectEditingContext context) {
		
		String typeName = "";
		
		if(!value.contains(":"))
		{
			return new String[0];
		}
		
		String[] tagValueType = value.split(":");
		if(tagValueType.length > 1)
		{
			typeName = tagValueType[1];
		}
		
		HashSet<String> proposals = new HashSet<String>();
		for (ScenarioDiagramTypes tmpType : Util.ScenarioDiagramTypes.values()) {
			if(tagValueType.length > 0 && (typeName.equals("") || tmpType.toString().startsWith(typeName.toUpperCase())))
			{
				proposals.add(tagValueType[0] + ":" + tmpType.toString());
			}
		}
		
		String[] result = proposals.toArray(new String[proposals.size()]);
		
		return result;
	}
}
